/**
 * Applications management
 */
var JolicloudApps = {	
		  
     /**
	 * Get the state of the applications on the system
	 * Returns a list of applications (hashes) with the attributes:
	 * name: string representing the name
	 * installed: string representing the installed version or none
	 * candidate: string representing the candidate version
	 */
	getState: function(applications, callback)
	{
		Jolicloud.getCommunicator().call("JolicloudApps", "getState", callback, applications);
	},
	
	/**
     * Listen to install/update/remove/upgrade actions
     * performed by the daemon.
     * @param {Object} callback
     * @param {Object} active If true, start listening, else stop listening (for this callback)
     */
    listenToActions: function(active, callback)
    {
        Jolicloud.getCommunicator().listen("JolicloudApps", "listenToActions", callback, active);
    },
    
    /**
     * Listen to install/update/remove/upgrade actions
     * performed by the daemon.
     * @param {Object} callback
     * @param {Object} active If true, start listening, else stop listening (for this callback)
     */
    
    listenToActionStatus: function(active, callback)
    {
        //alert("listenToActionStatus2");
        Jolicloud.getCommunicator().listen("JolicloudApps", "listenToActionStatus", callback, active);
    },
    
    
    /**
     * Listen to new/disappeared possible upgrades
     * @param {Object} callback
     * @param {Object} active If true, start listening, else stop listening (for this callback)
     */
    listenToUpgrades: function(active, callback)
    {
        Jolicloud.getCommunicator().listen("JolicloudApps", "listenToUpgrades", callback, active);
    },
    
    /**
     * Listen to reboot needed signals
     * @param {Object} callback
     * @param {Object} active If true, start listening, else stop listening (for this callback)
     */
    listenToRebootNeeded: function(active, callback)
    {
        Jolicloud.getCommunicator().listen("JolicloudApps", "listenToRebootNeeded", callback, active);
    },
    
    /**
	 * Get available upgrades on the system
	 * Returns a list of applications (hashes) with the attributes:
	 * name: string representing the name
	 * installed: string representing the installed version or none
	 * candidate: string representing the candidate version
	 */
	getUpgrades: function(callback)
	{
		Jolicloud.getCommunicator().call("JolicloudApps", "getUpgrades", callback);
	},
	
	/**
	 * Updates the local cached list of applications
	 */
	update: function(callback)
	{
		Jolicloud.getCommunicator().call("JolicloudApps", "update", callback);
	},
	
	/**
	 * Install applications
	 * Returns the list of applications names really installed
	 */
	install: function(applications, callback)
	{
		Jolicloud.getCommunicator().call("JolicloudApps", "install", callback, applications);
	},
	
	/**
	 * Upgrade applications, if no list given, upgrades all upgradeable applications
	 * Returns the list of applications names really upgraded
	 */
	upgrade: function(applications, callback)
	{
		Jolicloud.getCommunicator().call("JolicloudApps", "upgrade", callback, applications);
	},
	
	/**
	 * Remove applications
	 * Returns the list of applications names really removed
	 */
	remove: function(applications, callback)
	{
		Jolicloud.getCommunicator().call("JolicloudApps", "remove", callback, applications);
	},
	
	/**
	 * Compares 2 strings representing applications versions
	 * Returns:
	 * -1 if version1 < version2
	 * 0 if version1 = version2
	 * 1 if version1 > version2
	 */
	compareVersions: function(version1, version2)
	{
		/**
		 * Splits a version string in smaller parts:
		 * ints or strings without digits
		 */
		function splitVersion(version)
		{
			var versionExp = /^(([0-9]+):)?([a-z0-9+:~.-]+?)(-([a-z0-9+:~.]+))?$/i
			var versionPartExp = /[0-9]+|[a-z+:~.]+/ig
			
			var splitVer = versionExp.exec(version);
			var fullSplitVer = [];
			
			if (typeof splitVer[2] != "undefined") 
				fullSplitVer.push(splitVer[2]);
			else 
				fullSplitVer.push("0");
			
			fullSplitVer = fullSplitVer.concat(splitVer[3].match(versionPartExp));
			
			fullSplitVer.push("~");
			
			if (typeof splitVer[5] != "undefined") 
				fullSplitVer = fullSplitVer.concat(splitVer[5].match(versionPartExp));
			
			return fullSplitVer;
		}
		
		/**
		 * Gets the weight to affect to a char
		 * to compare in a debian version string
		 */
		function getCharWeight(ch)
		{
			if (ch == null || ch == "") 
				return 0;
			else if (ch == "~") 
				return -10;
			else if (ch.match(/[a-z]/i)) 
				return 10;
			else if (ch.match(/[+:.-]/)) 
				return 20;
			return 10;
		}
		
		/**
		 * Compares 2 debian version strings
		 */
		function compareStrings(string1, string2)
		{
			var inverter = 1;
			if (string1.length < string2.length) 
			{
				var temp = string2;
				string2 = string1;
				string1 = temp;
				inverter *= -1;
			}
			for (var i = 0; i < string1.length; i++) 
			{
				var c1 = string1[i];
				var c2 = typeof string2[i] != "undefined" ? string2[i] : null;
				var cw1 = getCharWeight(c1);
				var cw2 = getCharWeight(c2);
				if (cw1 > cw2) 
					return inverter * 1;
				else if (cw1 < cw2) 
					return inverter * -1;
				else if (c1 > c2) 
					return inverter * 1;
				else if (c1 < c2) 
					return inverter * -1;
			}
			return 0;
		}
		
		/**
		 * Compares a part of a debian version (integer of string withour digits)
		 */
		function compareVersionParts(part1, part2)
		{
			var part1Number = part1 != null && part1.match(/^[0-9]+$/);
			var part2Number = part2 != null && part2.match(/^[0-9]+$/);
			if (part1Number && part2Number) 
				if (parseInt(part1) > parseInt(part2)) 
					return 1;
				else if (parseInt(part1) < parseInt(part2)) 
					return -1;
				else 
					return 0;
			else if (part1Number) 
				return 1;
			else if (part2Number) 
				return -1;
			else 
				return compareStrings(part1, part2);
		}
		
		//We check for empty versions
		if ((!version1) && version2) 
			return -1;
		if (version1 && (!version2)) 
			return 1;
		if ((!version1) && (!version2)) 
			return 0;
		
		//We invert versions to have the longest in 1
		var inverter = 1;
		var splitVer1 = splitVersion(version1);
		var splitVer2 = splitVersion(version2);
		if (splitVer1.length < splitVer2.length) 
		{
			var temp = splitVer2;
			splitVer2 = splitVer1;
			splitVer1 = temp;
			inverter *= -1;
		}
		//we compare versions
		for (var i = 0; i < splitVer1.length; i++) 
		{
			var part1 = splitVer1[i];
			var part2 = typeof splitVer2[i] != "undefined" ? splitVer2[i] : "";
			var comp = compareVersionParts(part1, part2)
			if (comp != 0) 
				return inverter * comp;
		}
		return 0;
	}
}

window.JolicloudApps = JolicloudApps;
