/**
 * Manages the communication between the webpage and the extension (on the webpage side)
 */
var JolicloudCommunicator = function()
{
	//Initialize the communication root XML element
	this._communicationNode = document.createElement('JolicloudCommunication');
	document.documentElement.appendChild(this._communicationNode);
	
	//Listens to extension replies
	this._communicationNode.addEventListener("JolicloudExtEvent", this._onExtensionReply.bind(this), false, true);
	
	//list of calls done
	this._calls = [];
	
	//list of saved callbacks
	this._callbacks = {};
}

JolicloudCommunicator.prototype = {
	/**
	 * Call a function of the extension
	 * Extra arguments are the arguments of the called function
	 */
	call: function(targetName, functionName, callback)
	{
		var event = document.createEvent('Events');
		event.initEvent('JolicloudWebpageEvent', true, false);
		var communicationElement = document.createElement('JolicloudCall');
		
		var args = new Array();
		for (var i = 3; i < arguments.length; i++) 
		{
			args.push(arguments[i]);
		}
		communicationElement.setAttribute('target', targetName);
		communicationElement.setAttribute('function', functionName);
		communicationElement.setAttribute('arguments', JolicloudUtils.toJson(args));
		communicationElement.setAttribute('callid', this._calls.length);
		
		this._calls[this._calls.length] = {
			callback: callback
		};
		
		this._communicationNode.appendChild(communicationElement);
		communicationElement.dispatchEvent(event);
	},
	
	/**
	 * Call a function to listen to an event
	 * @param {Object} targetName
	 * @param {Object} functionName
	 * @param {Object} callback
	 * @param {Object} active If true, start listening, else stop listening (for this callback)
	 */
	listen: function(targetName, functionName, callback, active)
	{
		//Generate an id from the callback
		if (!this._callbacks[targetName])
			this._callbacks[targetName] = {};
		if (!this._callbacks[targetName][functionName])
			this._callbacks[targetName][functionName] = [];			
		var id = this._callbacks[targetName][functionName].indexOf(callback);
		if (id == -1)
		{
			id =this._callbacks[targetName][functionName].length;
			this._callbacks[targetName][functionName].push(callback);
		}
		else if (!active)
		{
			this._callbacks[targetName][functionName][id] = null;
		}
		this.call(targetName, functionName, callback, id, active);
	},
	
	/**
	 * Called when the extension replies
	 */
	_onExtensionReply: function(event)
	{
		var callElement = event.target;
		var callId = callElement.getAttribute('callid');
		var callback = this._calls[callId].callback;
		if (callback == null) 
			return;
		//can we delete this callback?
		if (callElement.getAttribute('last') == 'true') 
			this._calls[callId] = null;
		//return value
		var returnValue = callElement.getAttribute('return');
		
		if (!returnValue) 
			returnValue = null;
		else 
			returnValue = JolicloudUtils.fromJson(returnValue);
		//error handling
		var errorValue = callElement.getAttribute('error');
		if (!errorValue) 
			errorValue = null;
		else 
		{
			returnValue = null;
			errorValue = new Error(errorValue);
		}
		//call the callback
		callback(returnValue, errorValue);
	}
}

window.JolicloudCommunicator = JolicloudCommunicator;
