/**
 * Utils for the extension
 */

var JolicloudUtils =
{
	/**
	 * Converts an object to JSON
	 */
	toJson: function( obj )
	{		
		//Use native object since Firefox 3.1
		if( typeof JSON != "undefined" )
			return JSON.stringify( obj );
			
		switch (typeof obj) {
			case 'object':
				if (obj) {
					var list = [];
					if ( typeof obj.length != "undefined" && typeof obj.indexOf != "undefined" && typeof obj.join != "undefined" && typeof obj.concat != "undefined" && typeof obj.slice != "undefined" ) {
						for (var i=0;i < obj.length;i++) {
							list.push( this.toJson(obj[i]) );
						}
						return '[' + list.join(',') + ']';
					} else {
						for (var prop in obj) {
							list.push('"' + prop + '":' + this.toJson(obj[prop]));
						}
						return '{' + list.join(',') + '}';
					}
				} else {
					return 'null';
				}
			case 'string':
				return '"' + obj.replace(/"/g, '\\"') + '"';
			case 'number':
			case 'boolean':
				return new String(obj);
			default:
				return 'null';
		}
	},
	
	/**
	 * Parse JSON securely,
	 * from http://code.google.com/p/json-sans-eval
	 */
	fromJson: function ( json )
	{
		//Use native object since Firefox 3.1
		if( typeof JSON != "undefined" )
			return JSON.parse( json );
		
		//Quick fix for simple value false
		json = '[' + json + ']';
			
		var number = '(?:-?\\b(?:0|[1-9][0-9]*)(?:\\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\\b)';
		var oneChar = '(?:[^\\0-\\x08\\x0a-\\x1f\"\\\\]|\\\\(?:[\"/\\\\bfnrt]|u[0-9A-Fa-f]{4}))';
		var string = '(?:\"' + oneChar + '*\")';

		// Will match a value in a well-formed JSON file.
		// If the input is not well-formed, may match strangely, but not in an unsafe
		// way.
		// Since this only matches value tokens, it does not match whitespace, colons,
		// or commas.
		var jsonToken = new RegExp(
			'(?:false|true|null|[\\{\\}\\[\\]]'
			+ '|' + number
			+ '|' + string
			+ ')', 'g');

		// Matches escape sequences in a string literal
		var escapeSequence = new RegExp('\\\\(?:([^u])|u(.{4}))', 'g');

		// Decodes escape sequences in object literals
		var escapes = {
			'"': '"',
			'/': '/',
			'\\': '\\',
			'b': '\b',
			'f': '\f',
			'n': '\n',
			'r': '\r',
			't': '\t'
		};
		function unescapeOne(_, ch, hex) {
			return ch ? escapes[ch] : String.fromCharCode(parseInt(hex, 16));
		}

		// A non-falsy value that coerces to the empty string when used as a key.
		var EMPTY_STRING = new String('');
		var SLASH = '\\';

		// Constructor to use based on an open token.
		var firstTokenCtors = { '{': Object, '[': Array };

		// Split into tokens
		var toks = json.match(jsonToken);
		// Construct the object to return
		var result;
		var tok = toks[0];
		if ('{' === tok) {
			result = {};
		} else if ('[' === tok) {
			result = [];
		} else {
			throw new Error(tok);
		}

		// If undefined, the key in an object key/value record to use for the next
		// value parsed.
		var key;
		// Loop over remaining tokens maintaining a stack of uncompleted objects and
		// arrays.
		var stack = [result];
		for (var i = 1, n = toks.length; i < n; ++i) {
			tok = toks[i];

			var cont;
			switch (tok.charCodeAt(0)) {
				default:  // sign or digit
					cont = stack[0];
					cont[key || cont.length] = +(tok);
					key = void 0;
					break;
				case 0x22:  // '"'
					tok = tok.substring(1, tok.length - 1);
					if (tok.indexOf(SLASH) !== -1) {
						tok = tok.replace(escapeSequence, unescapeOne);
					}
					cont = stack[0];
					if (!key) {
						if (cont instanceof Array) {
							key = cont.length;
						} else {
							key = tok || EMPTY_STRING;  // Use as key for next value seen.
							break;
						}
					}
					cont[key] = tok;
					key = void 0;
					break;
				case 0x5b:  // '['
					cont = stack[0];
					stack.unshift(cont[key || cont.length] = []);
					key = void 0;
					break;
				case 0x5d:  // ']'
					stack.shift();
					break;
				case 0x66:  // 'f'
					cont = stack[0];
					cont[key || cont.length] = false;
					key = void 0;
					break;
				case 0x6e:  // 'n'
					cont = stack[0];
					cont[key || cont.length] = null;
					key = void 0;
					break;
				case 0x74:  // 't'
					cont = stack[0];
					cont[key || cont.length] = true;
					key = void 0;
					break;
				case 0x7b:  // '{'
					cont = stack[0];
					stack.unshift(cont[key || cont.length] = {});
					key = void 0;
					break;
				case 0x7d:  // '}'
					stack.shift();
					break;
			}
		}
		return result[0];
	}
}

Function.prototype.bind = function(){ 
  var fn = this, args = Array.prototype.slice.call(arguments), object = args.shift(); 
  return function(){ 
    return fn.apply(object, 
      args.concat(Array.prototype.slice.call(arguments))); 
  }; 
};

window.JolicloudUtils = JolicloudUtils;
