/**
 * Jolicloud account management
 */
var JolicloudAccount = {
	
	/**
	 * Listening to the count of actions received by the user
	 * @param {Object} session
	 * @param {Object} callId
	 * @param {Object} id unique identifier of this call
	 * @param {Object} active if true, start listening, if false, stop listening
	 */
	listenToActionsCount: function(session, callId, id, active)
	{		
		JolicloudDaemon.listenAndSend(session, callId, 'AccountActionsCountListeners' + id, active, '/AccountManager', 'ActionsCountChanged',
		function(result)
		{
			if (result == null)
				result = 0;
			else
				result = result[0];
			if (result == null)
				result = 0;
			return result;
		});
	},

	/**
	 * Get the actions count
	 */
	getActionsCount: function(session, callId)
	{
		JolicloudDaemon.callAndSend(session, callId, '/AccountManager', 'GetActionsCount', [],
		function(result)
		{
			if (result == null)
				result = 0;
			return result;
		});
	},

	/**
	 * Set the actions until now as read
	 */
	setActionsRead: function(session, callId, readTime)
	{
		JolicloudDaemon.callAndSend(session, callId, '/AccountManager', 'SetActionsRead', [readTime]);
	},

	/**
	 * Get the computer UUID
	 */
	getComputerUUID: function(session, callId)
	{
		JolicloudDaemon.callAndSend(session, callId, '/AccountManager', 'GetComputerUUID');
	},
	
	/**
	 * Get the currently saved jolicloud token, if any
	 */
	getJolicloudToken: function(session, callId)
	{
		JolicloudDaemon.callAndSend(session, callId, '/AccountManager', 'GetJolicloudToken', [],
		function(response)
		{
			return {'key': response[0], 'secret': response[1]};
		});
	},
	
	/**
	 * Refresh the jolicloud token in the daemon
	 */
	refreshJolicloudToken: function(session, callId)
	{
		JolicloudDaemon.callAndSend(session, callId, '/AccountManager', 'RefreshJolicloudToken');
	},
	
	/**
	 * Set the currently saved jolicloud token
	 */
	setJolicloudToken: function(session, callId, token_key, token_secret)
	{
		if (!token_key || !token_secret)
		{
			token_key = '';
			token_secret = '';
		}
		if (typeof(token_key) != 'string')
			token_key = '' + token_key;
		if (typeof(token_secret) != 'string')
			token_secret = '' + token_secret;
		JolicloudDaemon.callAndSend(session, callId, '/AccountManager', 'SetJolicloudToken', [token_key, token_secret]);
	},
}
