/**
 * Applications management
 * For now the only backend is APT
 */
var JolicloudApps = {
        
     /**
	 * Get the state of the applications on the system
	 */
	getState: function(session, callId, packages)
	{
		if (typeof packages == "string")
			packages = new Array(packages);
		if (packages.length == 0)
			return [];
		JolicloudDaemon.callAndSend(session, callId, '/SoftwareManager', 'GetState', [packages]);
	},
	
	/**
     * Listening to the actions done on the apps
     * @param {Object} session
     * @param {Object} callId
     * @param {Object} id unique identifier of this call
     * @param {Object} active if true, start listening, if false, stop listening
     */
    listenToActions: function(session, callId, id, active)
    {       
        JolicloudDaemon.listenAndSend(session, callId, 'AppsActionsListeners' + id, active, '/SoftwareManager', 'ActionProcessed',
        function(result)
        {
            return {
                action: result[0],
                packages: result[1],
                success: result[2],
                error: result[3]
            };
        });
    },
    
    /**
     * Listening to the actions done on the apps
     * @param {Object} session
     * @param {Object} callId
     * @param {Object} id unique identifier of this call
     * @param {Object} active if true, start listening, if false, stop listening
     */
   
    listenToActionStatus: function(session, callId, id, active)
    {       
        JolicloudDaemon.listenAndSend(session, callId, 'AppsActionsListeners' + id, active, '/SoftwareManager', 'ActionStatusChange',
        function(result)
        {
            return {
                action: result[0],
                status: result[1],
                progress: result[2],
                packages: result[3],
                success: result[4],
                error: result[5]
            };
        });
    },
    
    
    /**
	 * Get the list of upgradable apps on the system
	 * @param {Object} session
	 * @param {Object} callId
	 */
	getUpgrades: function(session, callId)
	{
		JolicloudDaemon.callAndSend(session, callId, '/SoftwareManager', 'GetUpgrades', null,
		function(result)
		{
			new_result = {
				system: [],
				apps: [],
				last_update_time: 0.0
			}
			if (result && result[0])
				new_result.system = result[0];
			if (result && result[1])
				new_result.apps = result[1];
			for (var i = 0; i < new_result.apps.length; i++)
			{
				var app = new_result.apps[i];
				app.category = {};
				app.pictures = {};
				for (var key in app)
				{
					if (key.substr(0, 9) == 'category_')
					{
						app.category[key.substr(9)] = app[key];
						delete app[key];
					}
					if (key.substr(0, 8) == 'picture_')
					{
						app.pictures[key.substr(8)] = app[key];
						delete app[key];
					}
				}
			}
			if (result && result[2] && result[2][0] && result[2][0]['last_update_time'])
				new_result.last_update_time = parseFloat(result[2][0]['last_update_time']) * 1000;
			return new_result;
		});
	},
		
	/**
	 * Listening to the possible upgrades
	 * @param {Object} session
	 * @param {Object} callId
	 * @param {Object} id unique identifier of this call
	 * @param {Object} active if true, start listening, if false, stop listening
	 */
	listenToUpgrades: function(session, callId, id, active)
	{
		JolicloudDaemon.listenAndSend(session, callId, 'AppsUpgradesListeners' + id, active, '/SoftwareManager', 'UpgradesChanged',
		function(result)
		{
			new_result = {
				system: [],
				apps: [],
				last_update_time: 0.0
			}
			if (result && result[0])
				new_result.system = result[0];
			if (result && result[1])
                new_result.apps = result[1];
			if (result && result[3])
                new_result.packages = result[3];
            for (var i = 0; i < new_result.apps.length; i++)
			{
				var app = new_result.apps[i];
				app.category = {};
				app.pictures = {};
				for (var key in app)
				{
					if (key.substr(0, 9) == 'category_')
					{
						app.category[key.substr(9)] = app[key];
						delete app[key];
					}
					if (key.substr(0, 8) == 'picture_')
					{
						app.pictures[key.substr(8)] = app[key];
						delete app[key];
					}
				}
			}
			if (result && result[2])
				new_result.last_update_time = parseFloat(result[2]) * 1000;
			return new_result;
		});
	},
	
	listenToRebootNeeded:function(session, callId, id, active)
    {
        JolicloudDaemon.listenAndSend(session, callId, 'AppsUpgradesListeners' + id, active, '/SoftwareManager', 'RebootNeeded',
        function(result)
        {
         return result;   
        });
    },
	
	/**
	 * Updates the local cached list of applications
	 */
	update: function(session, callId)
	{
		JolicloudDaemon.callAndSend(session, callId, '/SoftwareManager', 'Update');
	},
	
	/**
	 * Installs applications
	 */
	install: function(session, callId, packages)
	{
		if (typeof packages == "string") 
			packages = new Array(packages);
		JolicloudDaemon.callAndSend(session, callId, '/SoftwareManager', 'Install', [packages]);
	},
	
	/**
	 * Upgrades applications
	 */
	upgrade: function(session, callId, packages)
	{
		if (typeof packages == "string") 
			packages = new Array(packages);
		JolicloudDaemon.callAndSend(session, callId, '/SoftwareManager', 'Upgrade', [packages]);
	},
	
	/**
	 * Remove applications
	 */
	remove: function(session, callId, packages)
	{
		if (typeof packages == "string") 
			packages = new Array(packages);
		JolicloudDaemon.callAndSend(session, callId, '/SoftwareManager', 'Remove', [packages]);
	}
}
