/**
 * Manages the communication between the webpage and the extension (on the extension side)
 */
var JolicloudCommunicator = function()
{
	document.addEventListener('JolicloudWebpageEvent', this._onWebpageCall, false, true);
}

JolicloudCommunicator.prototype = {
	/**
	 * Called when the webpage calls a
	 * Jolicloud function
	 */
	_onWebpageCall: function(event)
	{
		//analyze the call
		var callElement = event.target;
		var callSession = Jolicloud.getSession(callElement.ownerDocument.defaultView);
		var callbackId = callElement.getAttribute('callid');
		var callId = callSession.registerCallback(
		function(result, error, last)
		{
			JolicloudCommunicator.prototype._callWebpage(callElement.parentNode, callbackId, result, error, last);
		});
		
		var targetName = callElement.getAttribute('target');
		var functionName = callElement.getAttribute('function');
		var args = callElement.getAttribute('arguments');
		if (args != null) 
			args = JolicloudUtils.fromJson(args);
		if (!args instanceof Array) 
			args = [];
		args.unshift(callId);
		args.unshift(callSession);
		
		//do we have all data we need?
		if (targetName != null && functionName != null && args != null) 
		{
			//call authorized? IMPORTANT SECURITY CHECK
			if (callSession.isAuthorizedAction(targetName, functionName)) 
			{
				object = eval(targetName);
				try 
				{
					var result = object[functionName].apply(object, args);
					if (result != null) 
						callSession.callbackCall(callId, result);
				} 
				catch (error) 
				{
					callSession.callbackError(callId, error.message);
				}
			}
			else 
			{
				callSession.callbackError(callId, 'NOT_AUTHORIZED_ERROR');
			}
		}
		else 
		{
			callSession.callbackError(callId, 'WRONG_CALL_DATA_ERROR');
		}
	},
	
	/**
	 * Send a reply or an event to the webpage
	 * @param {Object} container Container of the event dispatcher elements
	 * @param {Object} callbackId Id of the callback to trigger
	 * @param {Object} result Result if any
	 * @param {Object} error Error if any
	 * @param {Object} last Last time this callback is triggered?
	 */
	_callWebpage: function(container, callbackId, result, error, last)
	{
		var targetDocument = container.ownerDocument;
		
		var element = targetDocument.createElement('jolicloudreply');
		element.setAttribute('callid', callbackId);
		if (!error)
			if (result != undefined)
				element.setAttribute('return', JolicloudUtils.toJson(result));
		else
			element.setAttribute('error', error);
		if (last)
			element.setAttribute('last', 'true');
		container.appendChild(element);

		var event = targetDocument.createEvent('Events');
		event.initEvent('JolicloudExtEvent', true, false);
		element.dispatchEvent(event);
	}
}
