var Jolicloud = {
	/**
	 * Configuration
	 */
	id: "jolicloud-bridge@jolicloud.org",
	allowedDomain: /^((.*\.)?jolicloud\.com|localhost)?$/,
	webpageAvailableInterface: {
		'Jolicloud': {
			'default': true,
			'actions': ['load']
		},
		'JolicloudApps': {
			'default': false,
			'actions': ['listenToRebootNeeded', 'dbus', 'dbusListen', 'getState', 'getUpgrades', 'upgradeAll', 'getPackageUpgrades', 'update', 'install', 'remove', 'upgrade', 'listenToActions', 'listenToActionStatus', 'listenToUpgrades']
		},
		'JolicloudAccount': {
			'default': false,
			'actions': ['getComputerUUID', 'getJolicloudToken', 'setJolicloudToken', 'refreshJolicloudToken', 'getActionsCount', 'setActionsRead', 'listenToActionsCount']
		}
	},
	
	/**
	 * Active sessions
	 */
	sessions: [],
	
	/**
	 * Loads the wanted libraries
	 */
	load: function(session, callId, libraries)
	{
		var loadedLibraries = [];
		if (typeof libraries == "string") 
			libraries = new Array(libraries);
		for (var i = 0; i < libraries.length; i++) 
		{
			if (typeof this.webpageAvailableInterface[libraries[i]] != "undefined" && !this.webpageAvailableInterface[libraries[i]]['default']) 
			{
				session.injectCode(JolicloudUtils.loadWebpageJSFile(libraries[i]));
				loadedLibraries.push(libraries[i]);
			} else {
			    throw "unknown Or illegal library: " + libraries[i];
			}
		}
		return loadedLibraries;
	},
	
	/**
	 * Get a session for a window. (Create it if needed)
	 * @param {Object} safeWindow
	 */
	getSession: function( safeWindow )
	{
		for( var i = 0; i < Jolicloud.sessions.length; i++ )
			if( Jolicloud.sessions[i].getSafeWindow() == safeWindow )
				return Jolicloud.sessions[i];
		var session = new JolicloudSession( safeWindow );
		Jolicloud.sessions.push( session );
		return session;
	},
	
	/**
	 * Delete a session for a window
	 * @param {Object} safeWindow
	 */
	deleteSession: function( safeWindow )
	{
		for( var i = 0; i < Jolicloud.sessions.length; i++ )
		{
			if( Jolicloud.sessions[i].getSafeWindow() == safeWindow )
			{
				Jolicloud.sessions.splice( i, 1 );
				return true;
			}
		}
		return false;
	}
}
