/**
 * Global Jolicloud session management
 */
var JolicloudSession = function(safeWindow)
{
	this._safeWindow = safeWindow;
	this._callbacks = [];
	this._listeners = [];
	//Place to put functions to stop processes running 
	//specifically for this session (for listeners...)
	this.externalDestroyers = {}
	//Prepares the sandbox to evaluate code in the page
	this._sandbox = new Components.utils.Sandbox(safeWindow);
	this._sandbox.window = this.getUnsafeWindow();
	this._sandbox.document = this.getUnsafeWindow().document;
}

JolicloudSession.prototype = {
	/**
	 * Checks if the window of this session has the rights
	 * to do at least a privileged action
	 */
	isAuthorizedWebpage: function()
	{
		return this._safeWindow.location.hostname.match(Jolicloud.allowedDomain);
	},
	
	/**
	 * Checks if the window of this session has the rights
	 * to do the action
	 */
	isAuthorizedAction: function(targetName, functionName)
	{
	    //Checks if the webpage is authorized
		if (!this.isAuthorizedWebpage()) 
			return false;
		//Checks if it is a known action
		if (typeof(Jolicloud.webpageAvailableInterface[targetName]) == 'undefined') 
			return false;
		return Jolicloud.webpageAvailableInterface[targetName]['actions'].indexOf(functionName) != -1;
	},
	
	/**
	 * Injects code in the window of this action
	 */
	injectCode: function(code)
	{
		Components.utils.evalInSandbox(code, this._sandbox);
	},
	
	/**
	 * Registers a callback
	 * @param {Object} callbackFunction Function to call to trigger the callback
	 */
	registerCallback: function(callbackFunction)
	{
		this._callbacks.push({
			'callback': callbackFunction
		});
		return this._callbacks.length - 1;
	},
	
	/**
	 * Sends an error to the callback
	 * @param {Object} callbackId
	 * @param {Object} errorValue
	 */
	callbackError: function(callbackId, errorValue)
	{
		if (this._callbacks[callbackId]) 
		{
			webpageElement = this._callbacks[callbackId].callback(null, errorValue, true);
			this._callbacks[callbackId] = null;
		}
	},
	
	/**
	 * Calls a callback
	 * @param {Object} callbackId
	 * @param {Object} returnValue
	 */
	callbackCall: function(callbackId, returnValue)
	{
		if (this._callbacks[callbackId])
		{
			webpageElement = this._callbacks[callbackId].callback(returnValue, null, true);
			this._callbacks[callbackId] = null;
		}
	},
	
	/**
	 * Calls a callback when an event is triggered, can be called many times
	 * @param {Object} callbackId
	 * @param {Object} eventData
	 */
	callbackEvent: function(callbackId, eventData)
	{
		if (this._callbacks[callbackId]) 
			webpageElement = this._callbacks[callbackId].callback(eventData, null, false);
	},
	
	/**
	 * Logs an element in the session firebug console (if exists)
	 */
	log: function(element)
	{
		if (typeof this.getUnsafeWindow().console != "undefined" && typeof this.getUnsafeWindow().console.log == "function") 
			this.getUnsafeWindow().console.log(element);
	},
	
	getSafeWindow: function()
	{
		return this._safeWindow;
	},
	
	getUnsafeWindow: function()
	{
		return this._safeWindow.wrappedJSObject;
	}
}
